@@ -51,6 +51,8 @@
 //!     .find();
 //! # }
 //! ```
+use theme::BASE_PATHS;
+
 use crate::cache::{CacheEntry, CACHE};
 use crate::theme::{try_build_icon_path, THEMES};
 use std::path::PathBuf;
@@ -73,21 +75,25 @@ mod theme;
 /// ])
 /// # }
 pub fn list_themes() -> Vec<&'static str> {
-    THEMES
+    let mut themes = THEMES
         .values()
+        .flatten()
         .map(|path| &path.index)
         .filter_map(|index| {
             index
                 .section(Some("Icon Theme"))
                 .and_then(|section| section.get("Name"))
         })
-        .collect()
+        .collect::<Vec<_>>();
+    themes.dedup();
+    themes
 }
 
 /// The lookup builder struct, holding all the lookup query parameters.
 pub struct LookupBuilder<'a> {
     name: &'a str,
     cache: bool,
+    force_svg: bool,
     scale: u16,
     size: u16,
     theme: &'a str,
@@ -174,6 +180,24 @@ impl<'a> LookupBuilder<'a> {
         self
     }
 
+    /// By default [`find`] will prioritize Png over Svg icon.
+    /// Use this if you need to prioritize Svg icons. This could be useful
+    /// if you need a modifiable icon, to match a user theme for instance.
+    ///
+    /// ## Example
+    /// ```rust
+    /// # fn main() {
+    /// use freedesktop_icons::lookup;
+    ///
+    /// let icon = lookup("firefox")
+    ///     .force_svg()
+    ///     .find();
+    /// # }
+    pub fn force_svg(mut self) -> Self {
+        self.force_svg = true;
+        self
+    }
+
     /// Execute the current lookup
     /// if no icon is found in the current theme fallback to
     /// `/usr/share/icons/hicolor` theme and then to `/usr/share/pixmaps`.
@@ -186,6 +210,7 @@ impl<'a> LookupBuilder<'a> {
         Self {
             name,
             cache: false,
+            force_svg: false,
             scale: 1,
             size: 24,
             theme: "hicolor",
@@ -210,32 +235,65 @@ impl<'a> LookupBuilder<'a> {
         }
 
         // Then lookup in the given theme
-        THEMES.get(self.theme).and_then(|icon_theme| {
-            let icon = icon_theme
-                .try_get_icon(self.name, self.size, self.scale)
-                .or_else(|| {
-                    // Fallback to the parent themes recursively
-                    icon_theme.inherits().into_iter().find_map(|parent| {
-                        THEMES.get(parent).and_then(|parent| {
-                            parent.try_get_icon(self.name, self.size, self.scale)
+        THEMES
+            .get(self.theme)
+            .or_else(|| THEMES.get("hicolor"))
+            .and_then(|icon_themes| {
+                let icon = icon_themes
+                    .iter()
+                    .find_map(|theme| {
+                        theme.try_get_icon(self.name, self.size, self.scale, self.force_svg)
+                    })
+                    .or_else(|| {
+                        // Fallback to the parent themes recursively
+                        let mut parents = icon_themes
+                            .iter()
+                            .flat_map(|t| t.inherits())
+                            .collect::<Vec<_>>();
+                        parents.dedup();
+                        parents.into_iter().find_map(|parent| {
+                            THEMES.get(parent).and_then(|parent| {
+                                parent.iter().find_map(|t| {
+                                    t.try_get_icon(self.name, self.size, self.scale, self.force_svg)
+                                })
+                            })
                         })
                     })
-                })
-                .or_else(|| {
-                    THEMES
-                        .get("hicolor")
-                        // Fallback to 'hicolor'
-                        .and_then(|hicolor| hicolor.try_get_icon(self.name, self.size, self.scale))
-                })
-                // Last chance, try to find the icon in "/usr/share/pixmaps"
-                .or_else(|| try_build_icon_path(self.name, "/usr/share/pixmaps"));
+                    .or_else(|| {
+                        THEMES.get("hicolor").and_then(|icon_themes| {
+                            icon_themes.iter().find_map(|theme| {
+                                theme.try_get_icon(self.name, self.size, self.scale, self.force_svg)
+                            })
+                        })
+                    })
+                    .or_else(|| {
+                        for theme_base_dir in BASE_PATHS.iter() {
+                            if let Some(icon) =
+                                try_build_icon_path(self.name, theme_base_dir, self.force_svg)
+                            {
+                                return Some(icon);
+                            }
+                        }
+                        None
+                    })
+                    .or_else(|| {
+                        try_build_icon_path(self.name, "/usr/share/pixmaps", self.force_svg)
+                    })
+                    .or_else(|| {
+                        let p = PathBuf::from(&self.name);
+                        if let (Some(name), Some(parent)) = (p.file_stem(), p.parent()) {
+                            try_build_icon_path(&name.to_string_lossy(), parent, self.force_svg)
+                        } else {
+                            None
+                        }
+                    });
 
-            if self.cache {
-                self.store(self.theme, icon)
-            } else {
-                icon
-            }
-        })
+                if self.cache {
+                    self.store(self.theme, icon)
+                } else {
+                    icon
+                }
+            })
     }
 
     #[inline]
@@ -324,7 +382,7 @@ mod test {
             .with_theme("Papirus")
             .find();
 
-        asserting!("Given the same input parameter, lookup should ouput be the same as linincon")
+        asserting!("Given the same input parameter, lookup should output be the same as linincon")
             .that(&wireshark)
             .is_some()
             .is_equal_to(lin_wireshark);
